/*
 * f_exp.c
 *
 * V = f_exp(x, y, z, ps, cs, sigma)
 *
 * Exponential Function f with correlation length sigma
 *
 * V[k*Nx*Ny + i*Ny + j] = f([x(i); y(j); z(k)])
 *
 */

#include <math.h>
#include <string.h>
#include <stdio.h>
#include "mex.h"

#define X_IN	prhs[0]
#define Y_IN    prhs[1]
#define Z_IN    prhs[2]
#define P_IN	prhs[3]
#define C_IN    prhs[4]
#define S_IN    prhs[5]
#define V_OUT	plhs[0]

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
    
    double *x, *y, *z, *ps, *cs, *V, sigma;
    unsigned int Nx, Ny, Nz, Np;
    
    if(nrhs != 6 || nlhs != 1) {
        mexErrMsgTxt("f_exp requires 6 input arguments and one output argument: \n \
                ==> V = f_exp(x, y, z, ps, cs, sigma).");
    }
    
    /* read variables */
    x = mxGetPr(X_IN);
    y = mxGetPr(Y_IN);
    z = mxGetPr(Z_IN);
    ps = mxGetPr(P_IN);
    cs = mxGetPr(C_IN);
    sigma = mxGetScalar(S_IN);
    
    if(fabs(sigma) < 1e-15) {
        mexErrMsgTxt("sigma is almost zero, please specify an appropriate correlation length.");
    }
    
    /* number of points */
    Nx = mxGetN(X_IN);
    Ny = mxGetN(Y_IN);
    Nz = mxGetN(Z_IN);
    Np = mxGetN(P_IN);
    
    /* prepare output data */
    V_OUT = mxCreateDoubleMatrix(Nx*Ny*Nz, 1, mxREAL);
    
    /* define pointers to the out values */
    V = mxGetPr(V_OUT);
    memset(V, 0, Nx*Ny*Nz*sizeof(double));
    
    double sigma_inv = 1.0/sigma;
    unsigned int i, j, k, l; /* loop indices */
    
    /* loop in the appropriate order */
    for(k = 0; k < Nz; k++) {
        for(i = 0; i < Nx; i++) {
            for(j = 0; j < Ny; j++) {
                for(l = 0; l < Np; l++) {
                    /* calculate the sum over all c[l]*f(x - p_l) */
                    V[k*Nx*Ny + i*Ny + j] += cs[l]*exp(-sigma_inv*sqrt( \
                            (x[i] - ps[3*l])  *(x[i] - ps[3*l]) + \
                            (y[j] - ps[3*l+1])*(y[j] - ps[3*l+1]) + \
                            (z[k] - ps[3*l+2])*(z[k] - ps[3*l+2])));
                }
                /* divide by sigma */
                V[k*Nx*Ny + i*Ny + j] *= sigma_inv;                
            }
        }
    }   
}
